"use client"

import React, { useContext, useState } from "react";
import { UserContext } from "../_components/user-context";
import { redirect } from "next/navigation";

import { AppShell, Burger, Group, Image, Text, LoadingOverlay, Avatar, UnstyledButton, Title, Skeleton, Center, Switch, NavLink, ScrollArea, Code } from '@mantine/core';
import { IconLogout, IconExclamationCircle, IconNotes, IconCategory, IconTicket, IconEye } from '@tabler/icons-react';
import { useDisclosure } from '@mantine/hooks';
import classes from './layout.module.css';
import { KccbTheme } from "../_themes/KccbTheme";
import { useMutation, useQuery, useQueryClient } from "@tanstack/react-query";
import Link from "next/link";
import { notifications } from "@mantine/notifications";
import axios, { AxiosError } from "axios";
import { useRouter } from "next/navigation";


async function logout() {
  try {
    await axios.post('/api/auth/logout');
  } catch (error: AxiosError | any) {
    console.log(error);
    notifications.show({
      color: 'red',
      title: error.code,
      message: error.message
    })
  }
}

export default function HomeLayout({
  children,
}: Readonly<{
  children: React.ReactNode;
}>) {

  const user = useContext(UserContext);
  if (user === null) redirect('/login');

  const router = useRouter();

  const queryClient = useQueryClient();

  const [burgerOpened, { toggle: toggleBurger }] = useDisclosure();
  const [activeNavItem, setActiveNavItem] = useState<string | null>(null);

  const logoutMutation = useMutation({
    mutationKey: ['logout'],
    mutationFn: logout
  })

  async function handleLogout(event: React.MouseEvent) {
    event.preventDefault();
    await logoutMutation.mutateAsync();
    await queryClient.refetchQueries({ queryKey: ['user'] })
  }

  let navItems: React.ReactNode[] = [];

  navItems.push(<>
    <NavLink key={'raised-ticket'} label={
      <>
        <IconTicket size={20} style={{ marginRight: '5px', transform: 'translateY(5px)' }} />
        <span style={{ fontSize: '18px' }}>Create Ticket </span>
      </>
    }
      active={activeNavItem === 'raised-ticket' || undefined}
      onClick={event => {
        event.preventDefault();
        setActiveNavItem('raised-ticket')
        router.push('/home/user/raised-ticket')
      }}
    />
    <NavLink key={'view-ticket'} label={<><IconEye size={20} style={{ marginRight: '8px', transform: 'translateY(5px)' }} />
      <span style={{ fontSize: '18px' }}>View Ticket </span>
    </>
    }
      active={activeNavItem === 'view-ticket' || undefined}
      onClick={event => {
        event.preventDefault();
        setActiveNavItem('view-ticket')
        router.push('/home/user/view-ticket');
      }}
    />
  </>
  )

  return (
    <>
      <LoadingOverlay
        visible={!user || logoutMutation.isPending || logoutMutation.isSuccess}
        overlayProps={{ backgroundOpacity: 1 }} />

      {user && <AppShell
        header={{ height: { base: 60, md: 70, lg: 80 } }}
        navbar={{
          width: { base: 200, md: 250, lg: 300 },
          breakpoint: 'sm',
          collapsed: { mobile: !burgerOpened },
        }}
        padding="lg"
      >
        <AppShell.Header>
          <Group h="100%" px="md" justify="space-between">
            <Group>
              <Burger opened={burgerOpened} onClick={toggleBurger} hiddenFrom="sm" size="sm" />
              <Image
                radius='md'
                h={50}
                w='auto'
                fit='contain'
                src={'https://encrypted-tbn0.gstatic.com/images?q=tbn:ANd9GcR-ZljkQXsXUP1-EwnIPZtVq_JWhwGUW7b0_eSMno-bag&s'}
              />
              <Title order={2} className={classes.title}>Customer Request Management</Title>
            </Group>
            {/* <Group>
              
              <UnstyledButton className={classes.user} onClick={() => {
                setActiveNavItem(null);
                router.push('/home/user');
              }}>
              </UnstyledButton>
            </Group> */}
            <Group>
              <Avatar radius="xl" src={null} alt={user.bank_account_no} color={KccbTheme.primaryColor}>
              </Avatar>
              <div style={{ flex: 1 }}>
                <Text size="sm" fw={500}> 
                  Account No: [{user.bank_account_no}]
                </Text>
                {/* <Text c='dimmed' size="sm" component="span">User: </Text>
                <Code c='dimmed'>{user.id}</Code> */}
              </div>
            </Group>
          </Group>
        </AppShell.Header>
        <AppShell.Navbar p="md">
          <Text>Menu</Text>
          <nav className={classes.navbar}>
            <div className={classes.navbarMain}>
              <ScrollArea>
                {navItems}
              </ScrollArea>
            </div>
          </nav>
          <div className={classes.footer}>
            <a className={classes.link} onClick={handleLogout}>
              <IconLogout className={classes.linkIcon} stroke={1.5} />
              <Text>Logout</Text>
            </a>
          </div>
        </AppShell.Navbar>
        <AppShell.Main>
          {children}
        </AppShell.Main>
      </AppShell>}
    </>
  )
}