"use client";

import {
  ActionIcon,
  Button,
  Container,
  Flex,
  Group,
  Modal,
  Space,
  Table,
  TextInput,
  Title,
  Tooltip,
} from "@mantine/core";
import { useDisclosure } from "@mantine/hooks";
import { IconEye, IconMessage, IconSearch } from "@tabler/icons-react";
import axios from "axios";
import { useEffect, useState } from "react";

interface Message {
  note: string;
}

interface Ticket {
  id: string;
  category_of_request: string;
  nature_of_request: string;
  created_date: string;
  status: string;
  message: Message[];
}

export default function Page() {
  const [opened, { open, close }] = useDisclosure(false);
  const [tickets, setTickets] = useState<Ticket[]>([]);
  const [searchQuery, setSearchQuery] = useState("");
  const [filteredTickets, setFilteredTickets] = useState<Ticket[]>([]);
  const [activeMessages, setActiveMessages] = useState<Message[] | null>(null);

  // Sort and filter tickets based on the time 
  useEffect(() => {
    const sortedTickets = [...tickets].sort(
      // (a, b) => new Date(b.created_date).getTime() - new Date(a.created_date).getTime()
      (a, b) => parseInt(b.id) - parseInt(a.id)
    );
    const results = sortedTickets.filter((ticket) =>
      ticket.category_of_request.toLowerCase().includes(searchQuery.toLowerCase())
    );
    setFilteredTickets(results);
  }, [searchQuery, tickets]);

  // Fetch tickets from API
  useEffect(() => {
    async function fetchTickets() {
      try {
        const response = await axios.get<Ticket[]>("/api/ticket");
        setTickets(response.data);
      } catch (error) {
        console.error("Failed to fetch tickets: ", error);
      }
    }
    fetchTickets();
  }, []);

  const handleOpenMessage = (messages: Message[]) => {
    setActiveMessages(messages);
    open();
  };

  //Add conditional row styles
  const getRowStyle = (status: string) => {
    switch (status.toLowerCase()) {
      case "resolved":
        return { backgroundColor: "#e0e3df", color: "#000"}; // Grey for closed
      default:
        return {};
    }
  };

  const rows = filteredTickets.map((ticket) => (
    <Table.Tr key={ticket.id} style={getRowStyle(ticket.status)}>
      <Table.Td>{ticket.id}</Table.Td>
      <Table.Td>{ticket.category_of_request}</Table.Td>
      <Table.Td>{ticket.nature_of_request}</Table.Td>
      <Table.Td>{ticket.created_date}</Table.Td>
      <Table.Td>{ticket.status}</Table.Td>
      <Table.Td>
        <Group>
          <ActionIcon
            variant="subtle"
            onClick={() => handleOpenMessage(ticket.message)}
          >
            <Tooltip label="Message">
              <IconMessage/>
            </Tooltip>
          </ActionIcon>
        </Group>
      </Table.Td>

      <Table.Td>
        <Group>
          <ActionIcon variant="subtle">
            <Tooltip label="Details of the Ticket">
              <IconEye/>
            </Tooltip>
          </ActionIcon>
        </Group>
      </Table.Td>
    </Table.Tr>
    
  ));
  return (
    <Container fluid>
      <Title order={3}>View Ticket</Title>
      <Space h="1rem" />
      <Flex align="center" justify="space-between" w={720}>
        <Flex align="center">
          <Title order={5}>Tickets</Title>
          <Space w="1rem" />
          <TextInput
            placeholder="Search"
            value={searchQuery}
            onChange={(event) => setSearchQuery(event.currentTarget.value)}
            radius="md"
            w={250}
            leftSection={<IconSearch size={16} />}
          />
        </Flex>
      </Flex>
      <Space h="1.5rem" />
      <Table
        w={720}
        stickyHeader
        stickyHeaderOffset={60}
        withTableBorder
        highlightOnHover
        horizontalSpacing="lg"
      >
        <Table.Thead>
          <Table.Tr>
            <Table.Th>Ticket ID</Table.Th>
            <Table.Th>Category</Table.Th>
            <Table.Th>Description</Table.Th>
            <Table.Th>Timestamp</Table.Th>
            <Table.Th>Status</Table.Th>
            <Table.Th>Message</Table.Th>
            <Table.Th>Summary</Table.Th>
          </Table.Tr>
        </Table.Thead>
        <Table.Tbody>{rows}</Table.Tbody>
      </Table>

      <Modal opened={opened} onClose={close} title="Message">
        <Flex direction="column" gap="sm">
          {activeMessages?.map((msg, index) => (
            <div
              key={index}
              style={{
                padding: "0.5rem",
                background: index % 2 === 0 ? "#f1f3f5" : "#dce6f2",
                borderRadius: "8px",
                maxWidth: "70%",
              }}
            >
              {msg.note}
            </div>
          ))}
          <Button onClick={close} mt="sm">
            Close
          </Button>
        </Flex>
      </Modal>
    </Container>
  );
}


